


;============================================================================================
; constants
;============================================================================================

MAPPER			= 0
PRG_COUNT		= 2
CHR_COUNT		= 1
PRG_RAM			= 0
MIRRORING		= %0000				; %0000 = horizontal, %0001 = vertical, %1000 = four-screen

			.include "file\constants.asm"



;============================================================================================
; variables
;============================================================================================

			.include "file\Ram.asm"



;============================================================================================
; iNES header
;============================================================================================

			.db "NES", $1A						; identification of the iNES header
			.db PRG_COUNT						; number of 16KB PRG-ROM pages
			.db CHR_COUNT						; number of 8KB CHR-ROM pages
			.db ((MAPPER & $0F) << 4) | MIRRORING | (PRG_RAM << 1)	; flags 6
			.db (MAPPER & $F0)					; flags 7
			.dsb 8, $00						; clear the remaining bytes



;============================================================================================
; prg 0
;============================================================================================

			.base $8000
			.include "file\data.asm"
			
IF $ > $BFFF
ERROR "Bank0 overflowed!"
ENDIF
			.org $C000



;============================================================================================
; prg 1
;============================================================================================

			.base $C000

; ------------------------------  NMI  -----------------------------
NMI:			sta nmi_A
			stx nmi_X
			sty nmi_Y

			lda PPU_STATUS
			jsr Sub_WriteVramBuffer
			lda #00
			sta OAM_ADDR					; reset PPU registers
			sta PPU_ADDR					;
			sta PPU_ADDR					;
			sta PPU_SCROLL					;
			sta PPU_SCROLL					;

			jsr Sub_ControllerStrobe			; check the controllers

			lda #>Shadow_OAM
			sta SPR_OAM

			dec nmi_FrameTimer
			lda CursorAnimation
			clc
			adc #08
			sta CursorAnimation
			
			lda #FALSE
			sta frameFinished

			lda nmi_A
			ldx nmi_X
			ldy nmi_Y
			rti


; ------------------------------  IRQ  -----------------------------
IRQBRK:			rti


; ------------------------------ RESET -----------------------------
RESET:			sei						;
			cld						;
			ldx #$40					; disable frame IRQ's
			stx $4017					;
			ldx #$FF					; reset the stack
			txs						;
			inx						; disable NMI/rendering
			stx PPU_CTRL					;
			stx PPU_MASK					;
			stx $4010					; disable DMC IRQ's

_wait1frame:		bit PPU_STATUS					; wait for one frame
			bpl _wait1frame					;

			txa						; zero out RAM
_zeroram:		sta $00, X					;
			sta $0100, X					;
			sta $0200, X					;
			sta $0300, X					;
			sta $0400, X					;
			sta $0500, X					;
			sta $0600, X					;
			sta $0700, X					;
			inx						;
			bne _zeroram					;

			ldx #02						; wait for another 2 frames
_wait2frame:		bit PPU_STATUS					;
			bpl _wait2frame					;
			dex						;
			bne _wait2frame					;

			jsr Sub_ClearVram				;
			
			; ------title screen------

			; -------main menu--------
			jmp Sub_Gameplay

			.include "file\routines.asm"
			.include "file\gameplay.asm"


IF $ > $FFF9
ERROR "Bank1 overflowed!"
ENDIF

			.org $FFFA
			.dw NMI
			.dw RESET
			.dw IRQBRK



;============================================================================================
; chr 0								00 01 02 03 04 05 06 07
;============================================================================================

			.incbin "file\chr\gameplay.bkg"
			.incbin "file\chr\gameplay.spr"


