;---------------------------------Sub_WriteVramBuffer---------------------------------
;	DataStruct:   <-+
;		  	|
;		  	+----byte0: VRAM addrHI
;		  	+----byte1: VRAM addrLO
;		  	+----byte2: Dlll llll
;		  	|           |||| ||||
;		  	|           |+++-++++-length of data stream
;		  	|           +---------(0 = vram++, 1 = vram+32)
;		 	+----byteN: data
;
;
;
;		when "data" == $FF
;		------------------
;
;		    format: $FF $ll $dd
;		                 ||  ++-data
;		                 ++-----length
;
Sub_WriteVramBuffer:	lda writeBufferToVram				; make sure that vram buffer
			beq _ExitWriteVramBuffer			;   should be processed

			lda vramBufferPtr				; check if anything has been
			cmp #$FF					;   written to the buffer
			beq _ExitWriteVramBuffer			;

			ldy #$FF
_checkMoreVramWrites:	cpy vramBufferPtr
			bcc _ExitWriteVramBuffer
			beq _ExitWriteVramBuffer

			lda VramBuffer, Y				; set the address in VRAM
			sta PPU_ADDR					;
			dey						;
			lda VramBuffer, Y				;
			sta PPU_ADDR					;
			dey						;

			; load the data structure and write it to vram
			lda VramBuffer, Y				; length of data stream
			bpl _setVramIncToOne				; if(bit7==0) jump to inc vram addr by 1
_setVramIncToThirtyTwo:	ldx #%10001100					; make vram increment by 32
			jmp _writeToVram				;
_setVramIncToOne:	ldx #%10001000					; make vram increment by 1
_writeToVram:		stx PPU_CTRL					;
			and #%01111111					; shift the bits back
			tax						;
_loopWriteMoreVram:	dey						; write all data to vram
			lda VramBuffer, Y				;
			cmp #RLE					;
			beq _isCtrl_FF					;
			sta PPU_DATA					;
			dex						;
			bne _loopWriteMoreVram				;
			dey						;
			jmp _checkMoreVramWrites			; jump back to top of routine and
									; check for more VRAM writes in the buffer
			
_isCtrl_FF:		dey						; process an rle stream
			dex						;
			lda VramBuffer, Y				;
			sta vramWriteLength				;
			dey						;
			dex						;
_loopCtrl_FF:		lda VramBuffer, Y				;
			sta PPU_DATA					;
			dec vramWriteLength				;
			bne _loopCtrl_FF				;
			dex						;
			bne _loopWriteMoreVram				;
									;
_ExitWriteVramBuffer:	ldy #$FF					; reset vram stack pointer
			sty vramBufferPtr				;
			lda ram_PPU2000					; restore the prior PPU state
			sta PPU_CTRL					;
			rts						;

;---------------------------------Sub_PushToVramBuffer---------------------------------
;
;	Description:	Writes data to VRAMBUFFER.
;
;	Arguments:	DataPtr       - points to location of data
;		A             - length of data
;	Return:	(none)
;
;	Routines Called:	(none)
;	Registers Affected:	A,X,Y
;	Variables Affected:	vramWriteLength
;		vramBufferPtr
;		VRAMBUFFER
Sub_PushToVramBuffer:	sta vramWriteLength
			lda #FALSE
			sta writeBufferToVram
			ldx vramBufferPtr
			ldy #00
_pushMoreDataToBuffer:	lda (DataPtr), Y
			sta VramBuffer, X
			iny
			dex
			dec vramWriteLength
			bne _pushMoreDataToBuffer
			stx vramBufferPtr
			lda #TRUE
			sta writeBufferToVram
			rts

;---------------------------------Sub_ControllerStrobe----------------------------------
;	Description:	Get the status of $4016 and stores to
;		P1Joypad in this format:
;
;		7  6   5     4    3  2  1  0
;		A, B, Sel, Start, U, D, L, R
;
;	Arguments:	(none)
;	Return Values:	P1Joypad
;
;	Routines Called:	(none)
;	Registers Affected:	A,Y
;	Variables Affected:	
;
Sub_ControllerStrobe:	ldx #00						; start on player 2 (X=1)
_nextJoypad:		lda Plyr_JoypadCur, X				; preserve the old joypad status
			sta Plyr_JoypadOld, X				;
			lda #01						; strobe the joypad
			sta PORT1					;
			lda #00						;
			sta PORT1					;
			sta Plyr_JoypadCur, X 				; reset the button variable
			ldy #8						; 8 buttons to load
_nextbutton:		lda PORT1, X					;
			and #01						;
			lsr						;
			rol Plyr_JoypadCur, X				;
			dey						;
			bne _nextbutton					;
			inx						; loop another joypad if needed
			cpx #$02					;
			bcc _nextJoypad					;
			rts						;

;-------------------------------Sub_ClearVram--------------------------------
;	Description:	
;
;	Arguments:	(none)
;	Return Values:	(none)
;
;	Routines Called:	(none)
;	Registers Affected:	A,X,Y
;	Variables Affected:	(none)
Sub_ClearVram:		ldx #$20					; clear name tables
			jsr Sub_ClearNameTable				;
			ldx #$2C					;
			jsr Sub_ClearNameTable				;
			jsr Sub_ClearSpritePage				; clear oam
			jsr Sub_ClearVRAMBuffer				; clear vram buffer
			rts						;

;-------------------------------Sub_ClearSpritePages--------------------------------
;	Description:	Sets the Y-coordinate of each sprite
;		on page 2 and 3 to #$F0.
;
;	Arguments:	(none)
;	Return Values:	(none)
;
;	Routines Called:	(none)
;	Registers Affected:	A,X,Y
;	Variables Affected:	(none)

Sub_ClearSpritePage:	ldx #00						;
			lda #SPRITENULL					;
_loop_clrSprPages:	sta Shadow_OAM, X				;
			inx						;
			inx						;
			inx						;
			inx						;
			bne _loop_clrSprPages				;
			rts						;

;;-------------------------------Sub_ClearVRAMBuffer--------------------------------
;	Description:	clear vram buffer	
;
;	Arguments:	(none)
;	Return Values:	(none)
;
;	Routines Called:	(none)
;	Registers Affected:	A,X,Y
;	Variables Affected:	(none)
Sub_ClearVRAMBuffer:	ldx #$FF
			stx vramBufferPtr
			inx
			txa
_loop_clrVramBuf:	sta VramBuffer, X
			inx
			bne _loop_clrVramBuf
			rts

;---------------------------------Sub_ClearNameTable----------------------------------
;	Description:	Clears a name table
;
;	Arguments:	X - high byte nametable address
;		    (20xx, 24xx, 28xx, 2Cxx)
;	Return Values:	(none)
;
;	Routines Called:	(none)
;	Registers Affected:	A,X,Y
;	Variables Affected:	(none)

Sub_ClearNameTable:	stx PPU_ADDR
			lda #$00
			sta PPU_ADDR

			ldy #04
			ldx #00
_sub_clrnat_loop:	sta PPU_DATA
			inx
			bne _sub_clrnat_loop
			dey
			bne _sub_clrnat_loop
			rts

;-----------------------------Sub_TurnOnScreen--------------------------------
;	Description:
;
;	Arguments:
;	Return Values:
;
;	Routines Called:
;	Registers Affected:
;	Variables Affected:

Sub_TurnOnScreen:	lda #%10001000					; turn some stuff on...
			sta PPU_CTRL					;
			sta ram_PPU2000					;
			lda #1						;     ...wait...
			jsr Sub_WaitNMI					;
			lda #%00011110					;     ....turn on the rest
			sta PPU_MASK					;
			sta ram_PPU2001					;
			rts						;


;-----------------------------Sub_WaitNMI--------------------------------
;	Description:	Kills time
;
;	Arguments:	A - number of NMIs
;	Return Values:	(none)
;
;	Routines Called:	(none)
;	Registers Affected:	A
;	Variables Affected:	NMIFrameTimer

Sub_WaitNMI:		sta nmi_FrameTimer
_loopsubwaitnmi:	lda nmi_FrameTimer
			bne _loopsubwaitnmi
			rts

;-----------------------------Cursor Functions--------------------------------
;	Description:	
;
;	Arguments:	(none)
;	Return Values:	(none)
;
;	Routines Called:	(none)
;	Registers Affected:	A, Y
;	Variables Affected:	CursorCoorY, CursorCoorX

Sub_IncCurY:		inc CursorCoorY					; change cursor coordinate
			.db $2C			; BIT			;
Sub_DecCurY:		dec CursorCoorY					;
			.db $2C			; BIT			;
Sub_IncCurX:		inc CursorCoorX					;
			.db $2C			; BIT			;
Sub_DecCurX:		dec CursorCoorX					;
									;
_func_updateCursor:	ldy #00						; check for Y coordinate
			lda CursorCoorY					;
			bpl _cursY_noUnderflow				;   -underflow
			lda (MenuPtr), Y				;
			jmp _cursY_finish				;     reset to max
_cursY_noUnderflow:	cmp (MenuPtr), Y				;   -overflow
			beq _cursY_finish				;
			bcc _cursY_finish				;
			lda #00						;     reset to 0
_cursY_finish:		sta CursorCoorY					; save Y coordinate
									;
			iny						; check for X coordinate
			lda CursorCoorX					;
			bpl _cursX_noUnderflow				;   -underflow
			lda (MenuPtr), Y				;
			jmp _cursX_finish				;     reset to max
_cursX_noUnderflow:	cmp (MenuPtr), Y				;   -overflow
			beq _cursX_finish				;
			bcc _cursX_finish				;
			lda #00						;     reset to 0
_cursX_finish:		sta CursorCoorX					; save X coordinate
									;
			rts						;

;-----------------------------
Sub_SetMenuPtrs:	asl
			tax
			lda menuStruct_Ptr, X
			sta MenuPtr
			lda menuStruct_Ptr + 1, X
			sta MenuPtr + 1
			lda #00
			sta CursorCoorY
			sta CursorCoorX
			sta CursorAnimation
			rts

menuStruct_Ptr:		.dw gameplay_CursorData

